/*
 * Copyright (c) 2011 by Intevation GmbH, Germany <info@intevation.de>
 *
 * This file is part of MXD2map.
 *
 * This program is free software under the LGPL (>=v2.1)
 * Read the file LICENCE.txt coming with the software for details
 * or visit http://www.gnu.org/licenses/ if it does not exist.
 *
 * MXD2map has been developed on behalf of the
 * Bundesamt fuer Seeschifffahrt und Hydrographie (BSH) in Hamburg
 * by Intevation GmbH.
 *
 * Authors:
 * Raimund Renkert <raimund.renkert@intevation.de>
 * Bjoern Schilberg <bjoern.schilberg@intevation.de>
 * Stephan Holl <stephan.holl@intevation.de>
 */

package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.HashLineSymbol;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import com.esri.arcgis.display.esriLineCapStyle;
import com.esri.arcgis.display.esriLineJoinStyle;

import org.w3c.dom.Element;
import java.io.IOException;

/**
 * Reads cartoline symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class HashLineSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(HashLineSymbolReader.class);

    /**
     * Private member.
     */
    private HashLineSymbol symbol;

    /**
     * Constructor with symbol.
     *
     * @param symbol The symbol used to display lines.
     */
    public HashLineSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("constructor()");
        if(symbol instanceof HashLineSymbol) {
            this.symbol = (HashLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a HashLineSymbol!");
        }
    }

    /**
     * Constructor with symbol.
     *
     * @param symbol The symbol used to display lines.
     */
    public HashLineSymbolReader(ILineSymbol symbol)
    throws Exception {
        logger.debug("constructor()");
        if(symbol instanceof HashLineSymbol) {
            this.symbol = (HashLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a HashLineSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read() {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        try {
            symbolElement.setAttribute("name", symbol.getNameString());
        }
        catch(IOException ioe) {
            logger.warn("Could not read name. Setting name to \"default\".");
            symbolElement.setAttribute("name", "default");
        }

        try {
            if(symbol.getColor() instanceof IRgbColor) {
                IRgbColor color = (IRgbColor)symbol.getColor();
                Color c = new Color (
                    color.getRed(),
                    color.getGreen(),
                    color.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
            else {
                RgbColor col = new RgbColor();
                col.setRGB(symbol.getColor().getRGB());
                Color c = new Color (
                    col.getRed(),
                    col.getGreen(),
                    col.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
        }
        catch(IOException ioe) {
            logger.warn("Could not read color.");
        }

        int cap;
        try {
            cap = symbol.getCap();
        }
        catch(IOException ioe) {
            logger.warn("Could not read line cap.");
            cap = -1;
        }
        String capStyle = "";
        switch(cap) {
            case esriLineCapStyle.esriLCSRound: capStyle = "round"; break;
            case esriLineCapStyle.esriLCSSquare: capStyle = "square"; break;
            default: break;
        }

        int join;
        try {
            join = symbol.getJoin();
        }
        catch(IOException ioe) {
            logger.warn("Could not read line join.");
            join = -1;
        }
        String joinStyle = "";
        switch(join) {
            case esriLineJoinStyle.esriLJSRound: joinStyle = "round"; break;
            case esriLineJoinStyle.esriLJSMitre: joinStyle = "miter"; break;
            default: break;
        }
        if(!capStyle.equals("")) {
            symbolElement.setAttribute("cap", capStyle);
        }
        if(!joinStyle.equals("")) {
            symbolElement.setAttribute("join", joinStyle);
        }

        try {
            symbolElement.setAttribute(
                "linestart",
                String.valueOf(symbol.getLineStartOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read line start offset.");
        }

        try {
            symbolElement.setAttribute(
                "miterlimit",
                String.valueOf(symbol.getMiterLimit()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read miter limit.");
        }

        try {
            symbolElement.setAttribute(
                "offset",
                String.valueOf(symbol.getOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read offset.");
        }

        try {
            symbolElement.setAttribute(
                "width",
                 String.valueOf(symbol.getWidth()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read width.");
        }

        try {
            ILineSymbol ls = symbol.getHashSymbol();
            readHashSymbol(ls, symbolElement);
        }
        catch(Exception e) {
            logger.warn(
                "Could not read HashSymbol." +
                " No fallback symbol defined.");
        }

        symbolElement.setAttribute("type", "line");
        symbolElement.setAttribute("style", "hash");

        return symbolElement;
    }

    /**
     * Read the hash line symbol.
     *
     * @param ls     The hash line symbol.
     * @param parent Parent DOM element.
     */
    private void readHashSymbol(ILineSymbol ls, Element parent)
    throws Exception {
        LineSymbolReader lsr = new LineSymbolReader ();
        if (lsr.canRead(ls)) {
            lsr.setSymbol(ls);
            lsr.setParent(parent);
            lsr.setUtil(util);
            lsr.read();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
